;	For 1 MB Maxflash with 128 banks of 8k each.
;	Cartmapper "42 - MaxFlash 1M / 8Mbit -older (bank127)
;	or "MaxFlash 1M / 8Mbit - new (bank0)" in Altirra.

;CARTCS	= $bffa 			;Start address vector, used if CARTFG has CARTFG_START_CART bit set
;CART	= $bffc				;Flag, must be zero for modules
;CARTFG	= $bffd				;Flags or-ed together, indicating how to start the module.
;CARTAD	= $bffe				;Initialization address vector

CARTFG_DIAGNOSTIC_CART = $80		;Flag value: Directly jump via CARTAD during RESET.
CARTFG_START_CART = $04			;Flag value: Jump via CARTAD and then via CARTCS.
CARTFG_BOOT = $01			;Flag value: Boot peripherals, then start the module.

;================================================================================
	opt h-				;Disable Atari COM/XEX file headers
;================================================================================

;--------------------------------------------------------------------------------
; includes:

	icl "../base/sys_equates.m65"

	icl "../base/sys_macros.m65"

;--------------------------------------------------------------------------------
; memory map

; remember that some move/copy macros use the last 6 bytes of page zero!

Vars_address			= 0
Macros_vars_address		= 250

;--------------------------------------------------------------------------------
; variables

	org Vars_address
	
m_samplePointer			org *+2
m_bankPages				org *+1
m_samplePointBuffer		org *+1

;--------------------------------------------------------------------------------
; constants

//PAL_VERSION			= 1

;================================================================================================================================================================
;	Bank 0 is the startup bank on newer Atarimax Maxflash 1MB cartridges only.
;	See bank 127 for the compatibility code for the older models.

	org $a000			; Bank 1
	;opt f+				; Activate fill mode

	.proc bank0			; Powerup code for new Atarimax 1 MB cartridges

	.proc init			; Cartridge initalization
	rts					; Only the minimum of the OS initialization is complete, you don't want to code here normally.
	.endp

;--------------------------------------------------------------------------------
	.proc start			; Cartridge start, RAM, graphics 0 and IOCB no for the editor (E:) are ready

	ClearSystem

	DisableBasic
	DisableOperativeSystem
	
	CopyMemory ClearAllMemory_cart_start, ClearAllMemory_target_start, ClearAllMemory_size
	
	lda #0		; return bank
	jsr ClearAllMemory
	
; only run in the correct system
	lda PAL
	and #14

.if .def PAL_VERSION
NO_PAL_loop
	bne NO_PAL_loop
.else
NO_NTSC_loop
	beq NO_NTSC_loop
.endif

	CopyMemory PlayTracksPDM_cart_start, PlayTracksPDM_target_start, PlayTracksPDM_size
	CopyMemory PlayTrack1_cart_start, PlayTrack1_target_start, PlayTrack1_size

	CopyMemory PlaySamplePDM_cart_start, PlaySamplePDM_target_start, PlaySamplePDM_size
	
	CopyMemory DisplayList, DisplayListRam, [DisplayListEnd-DisplayList]
	CopyMemory DisplayListData, DisplayListDataRam, [DisplayListDataEnd-DisplayListData]
	CopyMemory AtariFont, AtariFontRam, 1024
	CopyMemory TabAudioLow, TabAudioLowRam, 256
	CopyMemory TabAudioHigh, TabAudioHighRam, 256

	
	VcountSync 1
	
	lda #14
	sta COLPF1
	lda #2
	sta COLPF2
	
	SetFontAddress AtariFontRam
	
	SetDisplayListAddress DisplayListRam

	lda #[DV_DMA_ON | DV_NARROW_PF]
	sta DMACTL


wait_loop
	lda VCOUNT
	sta WSYNC
	sta COLBK
	
	lda SKSTAT
	and #4			// a key pressed
	bne wait_loop
	
	lda KBCODE
	cmp #31				// key "1"
	beq play_sample1
	
	jmp wait_loop

	
play_sample1
	lda #0
	sta DMACTL
	sta COLBK

	
	// lda #SAMPLE1_FIRST_BANK
	// sta PlaySamplePDM.FirstBankSMAddress+1
	// lda #[SAMPLE1_FIRST_BANK + SAMPLE1_CART_BANKS]
	// sta PlaySamplePDM.LastBankSMAddress+1
	// lda #0		// return bank
	// jsr PlaySamplePDM_target_start
	

	lda #0		// return bank
	jsr PlayTracksPDM_target_start
	
	
	VcountSync 1
	
	lda #[DV_DMA_ON | DV_NARROW_PF]
	sta DMACTL
	
	jmp wait_loop

	
	.endp

;================================================================================
ClearAllMemory_target_start = $300

	.proc ClearAllMemory, ClearAllMemory_target_start
	pha				; save return bank
	
	sta $d580		; disable all cart banks (enable normal ram)
	
	SetMemory (1*1024) (51*1024) 0
	SetMemory (54*1024) (10*1024) 0

	pla
	tax
	sta $d500,x		; enable return bank
	
	rts
	.endp
	
ClearAllMemory_size = .len ClearAllMemory
ClearAllMemory_cart_start = .adr ClearAllMemory

;================================================================================
	icl "PDM_15K_track.asm"
	
	icl "PDM_15K.asm"
	
;================================================================================
DisplayListRam = $E00
DisplayList
	:8 .byte DL_BLANK_8
	
	.byte GM_CHAR_G0+DL_LMS_MASK
	.word DisplayListDataRam
	.byte DL_BLANK_8
	.byte DL_BLANK_8
	.byte DL_BLANK_8
	.byte GM_CHAR_G0
	
	.byte DL_JVB
	.word DisplayListRam
DisplayListEnd

DisplayListDataRam = $F00
DisplayListData
	.sb " 1 - MegaMix PDM 8bits 15720Hz  "
	//.sb " 1 - Song 2, PDM 8bits 15720Hz  "
	.sb "  Press a # to play that song   "
DisplayListDataEnd

AtariFontRam = $1000
AtariFont
	ins "data\atari.fnt"

TabAudioLowRam = $1400
TabAudioLow
    :256 .byte $A0|[#&$F]		// $a0,a1,a2..af, $a0,a1,a2..af, ... $a0,a1,a2..af

TabAudioHighRam = $1500
TabAudioHigh
    :256 .byte $10|[#>>4]		// $10,10,10..10, $11,11,11..11, ... $1f,1f,1f..1f
	
;================================================================================
	.if * > $bff4
		.error "First bank overwrite final data!"
	.endif
;================================================================================
	.print "------------------------------------------------------------"
	.print "Bank 0 area: [", bank0, ", ", *-1, "]"
	.print " "
;================================================================================

	opt f+				; Activate fill mode
	org $bff4			; Powerup code for old Atarimax 1 MB cartridges

	.proc powerup
	lda $d500			; Dummy, will never be executed, see bank 127
	jmp start			; Continuation from bank 127				
	.endp

	org $bffa			; Cartridge control block
	
	.word start			; CARTCS
	.byte 0				; CART
	.byte CARTFG_START_CART		; CARTFG
	.word init			; CARTAD
	
	.endp

	opt f-				; Deactivate fill mode, so ORG can be changed again

;================================================================================================================================================================
SampleCartBuffer = $A000

.if .def PAL_VERSION

SAMPLE1_FIRST_BANK = 1
SAMPLE1_CART_BANKS = 120
SAMPLE1_END_BANK_SIZE = $1782			// low byte must be different than 0!
	//ins "data/mix2p_15k_ms_pwm.raw"
	ins "data/mix2p_15k_ms_pdm.raw"
	
.else

SAMPLE1_FIRST_BANK = 1
SAMPLE1_CART_BANKS = 121
SAMPLE1_END_BANK_SIZE = $14FB			// low byte must be different than 0!
	//ins "data/mix2_15k_ms_pwm.raw"
	ins "data/mix2_15k_ms_pdm.raw"

.endif

/*.if .def PAL_VERSION

SAMPLE1_FIRST_BANK = 1
SAMPLE1_CART_BANKS = 61
SAMPLE1_END_BANK_SIZE = $1DFF			// low byte must be different than 0!
	ins "data/r_pal_15k_ms_pdm.raw"
	
.else

SAMPLE1_FIRST_BANK = 1
SAMPLE1_CART_BANKS = 62
SAMPLE1_END_BANK_SIZE = $CFF			// low byte must be different than 0!
	ins "data/r_ntsc_15k_ms_pdm.raw"

.endif*/

;================================================================================================================================================================
;	Including a ready to use binary file with "INS", or part thereof will be much faster

DUMMY_BANKS = (128 - 2) - (SAMPLE1_CART_BANKS)

	.proc other_banks
	.rept DUMMY_BANKS

	org $a000
	opt f+

	.sav 8192
	
	opt f-

	.endr
	.endp
	
;================================================================================================================================================================
;	Special startup on bank 127 because older Atarimax Maxflash 1MB cartridges have start bank 127
;	Newer cartridges have start bank 0, but you should keep this code for compatibility with older hardware.

	org $a000			; Bank 127
	opt f+
	
	.proc bank_127		; Powerup code for old Atarimax 1 MB cartridges
	
	org $bff4

	.proc powerup
	lda $d500			; Activate bank 0
	jmp *				; Dummy, will never be executed, see bank 0
	.endp

	org $bffa			; Cartridge control block
	
	.word powerup                   ; CARTCS
	.byte 0                         ; CART
	.byte CARTFG_START_CART         ; CARTFG
	.word powerup                   ; CARTAD

	.endp
	
	opt f-

